from __future__ import annotations

from enum import StrEnum, auto


class Event(StrEnum):
  adPlaying = auto()
  autoplayUpNext = auto()
  nowAutoplaying = auto()
  onAdStateChange = auto()
  onAudioTrackChanged = auto()
  onAutoplayModeChanged = auto()
  onHasPreviousNextChanged = auto()
  onPlaylistModeChanged = auto()
  onSubtitlesTrackChanged = auto()
  onVideoQualityChanged = auto()
  onVolumeChanged = auto()
  playlistModified = auto()

  loungeScreenDisconnected = auto()
  noop = auto()
  loungeStatus = auto()
  nowPlaying = auto()
  onStateChange = auto()


class MsgType(StrEnum):
  REFRESH = auto()
  PAIR = auto()
  LINK = auto()

  PAIRED = auto()
  LINKED = auto()

  BEGIN_AD = auto()
  END_AD = auto()
  SKIP_AD = auto()
  MUTE_AD = auto()

  GOT_ART = auto()
  GOT_TITLE = auto()
  GOT_AUTH_STATE = auto()
  LOAD_AUTH_STATE = auto()

  CONNECTED = auto()
  DISCONNECTED = auto()

  PREVIOUS = auto()
  NEXT = auto()
  PLAY = auto()
  PAUSE = auto()
  STOP = auto()
  SET_VOLUME = auto()
  SEEK = auto()

  PAUSED = auto()
  PLAYING = auto()
  STOPPED = auto()
  BUFFERING = auto()
  STARTING = auto()

  AUTOPLAY_UP_NEXT = auto()
  LOUNGE_STATUS = auto()
  NOW_AUTOPLAYING = auto()
  NOW_PLAYING = auto()
  AD_STATE_CHANGE = auto()
  AUDIO_TRACK_CHANGE = auto()
  AUTOPLAY_MODE_CHANGE = auto()
  HAS_PREV_NEXT_CHANGE = auto()
  PLAYLIST_MODE_CHANGE = auto()
  STATE_CHANGE = auto()
  SUBTITLES_TRACK_CHANGE = auto()
  VIDEO_QUALITY_CHANGE = auto()
  VOLUME_CHANGE = auto()
  PLAYLIST_MODIFIED = auto()


from __future__ import annotations

from json import dumps, loads
from typing import Self, TypedDict

from pydantic import BaseModel

from .enums import MsgType


type JsonVal = bool | int | float | str | Json | list | None
type Json = dict[str, JsonVal]


class MessageJson(TypedDict):
  type: MsgType
  content: JsonVal


class MetadataJson(TypedDict):
  state: int
  currentTime: float
  duration: float
  videoId: str


class WithJson(BaseModel):
  @property
  def json(self) -> Json:
    return self.model_dump()

  @property
  def jsons(self) -> str:
    return dumps(self.json)

  @classmethod
  def from_json(cls, json: str) -> Self:
    return cls(**loads(json))


class Message(WithJson):
  type: MsgType
  content: JsonVal | None = None


class PlaybackData(WithJson):
  state: int
  currentTime: float
  videoId: str
  duration: float

  @staticmethod
  def from_event(state: 'PlaybackState') -> PlaybackData:
    return PlaybackData(
      state=state.state,
      currentTime=state.currentTime,
      videoId=state.videoId,
      duration=state.duration
    )
